<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Subscriber;


use Cybersponse\CoreBundle\Provider\CacheableInterface;
use Cybersponse\ActorsBundle\Entity\Actor;
use Dunglas\ApiBundle\Event\DataEvent;
use Dunglas\ApiBundle\Event\Events;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class UserCacheSubscriber implements EventSubscriberInterface
{
    /** @var CacheableInterface[] */
    protected $caches = [];

    /**
     * @param CacheableInterface $cache
     */
    public function addCache($cache)
    {
        $this->caches[] = $cache;
    }

    /**
     * @inheritdoc
     */
    public static function getSubscribedEvents()
    {
        return [
            Events::POST_UPDATE => ['clearUserPermissionsCache', 0],
            Events::POST_DELETE => ['clearUserPermissionsCache', 0]
        ];
    }

    /**
     * @param DataEvent $dataEvent
     */
    public function clearUserPermissionsCache(DataEvent $dataEvent)
    {
        $object = $dataEvent->getData();

        if ($object instanceof Actor && !is_null($object->getUserId())) {
            foreach ($this->caches as $cache) {
                $cache->deleteFromCache($object->getUserId());
            }
        }
    }
}