<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CustomerTenantBundle\Controller;

use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Serializer\Serializer;
use Dunglas\ApiBundle\Exception\DeserializationException;
use Dunglas\ApiBundle\Exception\ExceptionInterface;
use Symfony\Component\Serializer\Exception\ExceptionInterface as SerializerExceptionInterface;
use Cybersponse\UtilsBundle\Utils;
use Dunglas\ApiBundle\Api\ResourceInterface;

class RouterController extends AuthorizationResourceController
{
    const MODULE_TYPE = "routers";
    const REPLICATION_ROUTINGKEY = 'route.crudhub.tenant.request.routerconfig';

    /**
     * @param Request $request
     * @return Response|static
     */
    public function getAction(Request $request)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_RETRIEVE_LIST, new DataEvent($resource, null));
        $blockPassword = Utils::isPasswordBlock($request);
        $data = $this->getCollectionData($resource, $request);
        if ($request->get($this->container->getParameter('api.collection.pagination.page_parameter_name')) &&
            0 === count($data)) {
            throw $this->createNotFoundException();
        }

        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::RETRIEVE_LIST, new DataEvent($resource, $data));

        return $this->getSuccessResponse($resource, $data, Response::HTTP_OK, [], ['request_uri' => $request->getRequestUri()], [], $blockPassword);
    }

    protected function getSuccessResponse(
        ResourceInterface $resource,
        $data,
        $status = 200,
        array $headers = [],
        array $additionalContext = [],
        array $__selectFields = [],
        $passwordBlock = false
    ) {
        $__selectFields = [];
        if ($__selectFields) {
            $__selectFields = array_unique(array_merge($__selectFields, ['id', '@id', '@type']), SORT_REGULAR);
        }
        $normalized_data = $this->get('serializer')->normalize(
            $data,
            self::NORMALIZER_METHOD,
            $resource->getNormalizationContext() + $additionalContext + ['required_attributes' => $__selectFields,
            'block_password' => $passwordBlock]
        );

        $response = new Response(
            $normalized_data,
            $status,
            $headers
        );
        if ($status == Response::HTTP_CREATED) {
            $iri = $this->get('api.iri_converter')->getIriFromItem($data);
            $response->headers->set('Location', $iri);
        }
        return $response;
    }

    public function getOnIdAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_RETRIEVE, new DataEvent($resource, null));
        $blockPassword = Utils::isPasswordBlock($request);
        $object = $this->findOrThrowNotFound($resource, $id, $request);

        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::RETRIEVE, new DataEvent($resource, $object));

        return $this->getSuccessResponse($resource, $object, Response::HTTP_OK, [], [], [], $blockPassword);
    }

    /**
     * @param Request $request
     * @return Response
     * @throws DeserializationException
     */
    public function postAction(Request $request)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $utilsObj = new Utils();
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_CREATE, new DataEvent($resource, null));
        $blockPassword = Utils::isPasswordBlock($request);
        $request_data = $request->getContent();
        try {
            $object = $this->get('serializer')->deserialize(
                $request_data,
                $resource->getEntityClass(),
                self::NORMALIZER_METHOD,
                $resource->getDenormalizationContext()
            );
        } catch (ExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        } catch (SerializerExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        }

        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $object));

        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            // Normalize Updated Object
            // Setting Relationship true for sending auditing component relationship data
            $existing_relationship = false;
            if ($request->query->has("\$relationships")) {
                $existing_relationship = true;
            }
            $request->query->set("\$relationships", true);
            // Validation succeed
            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE, new DataEvent($resource, $object));
            /** @var Serializer $serializer */
            $serializer = $this->get('serializer');
            $updatedObject = $serializer->normalize(
                $object,
                self::NORMALIZER_METHOD,
                $resource->getNormalizationContext() + ['block_password' => false]);
            if (!$existing_relationship) {
                $request->query->remove("\$relationships", true);
            }
            $data = $this->prepareChangeData($updatedObject, (object)null, 'Create');

            $utilsObj->sendData($this, [$data]);

            $routerDiffObj = $utilsObj->prepareReplicationDiffData($updatedObject, null);
            $routerData = $utilsObj->prepareReplicationData($routerDiffObj, 'POST', null, null);
            $utilsObj->sendReplicationData($this, $routerData, self::REPLICATION_ROUTINGKEY);

            return $this->getSuccessResponse($resource, $object, Response::HTTP_CREATED, [], [], [], $blockPassword);
        }
        return $this->getErrorResponse($violations);
    }

    public function putAction(Request $request, $id)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $utilsObj = new Utils();
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE, new DataEvent($resource, null));
        $blockPassword = Utils::isPasswordBlock($request);
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE_DESERIALIZE, new DataEvent($resource, $object));

        $context = $resource->getDenormalizationContext();
        $context['object_to_populate'] = $object;
        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $existing_relationship = false;
        if ($request->query->has("\$relationships")) {
            $existing_relationship = true;
        }
        $request->query->set("\$relationships", true);
        $preUpdatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);
        $request_data = $request->getContent();
        try {
            $object = $serializer->deserialize(
                $request_data,
                $resource->getEntityClass(),
                self::NORMALIZER_METHOD,
                $context
            );
        } catch (ExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        } catch (SerializerExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        }

        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE_VALIDATION, new DataEvent($resource, $object));

        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            // Validation succeed
            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE, new DataEvent($resource, $object, $preUpdatedObject));
            $updatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext());
            $data = $this->prepareChangeData($updatedObject, $preUpdatedObject, 'Update', json_decode($request_data));

            $utilsObj->sendData($this, [$data]);

            $routerDiffObj = $utilsObj->prepareReplicationDiffData($updatedObject, $preUpdatedObject);
            $routerData = $utilsObj->prepareReplicationData($routerDiffObj, 'PUT', null, null);
            $utilsObj->sendReplicationData($this, $routerData, self::REPLICATION_ROUTINGKEY);

            if (!$existing_relationship) {
                $request->query->remove("\$relationships", true);
            }
            return $this->getSuccessResponse($resource, $object, Response::HTTP_OK, [], [], [], $blockPassword);
        }
        return $this->getErrorResponse($violations);
    }

    public function deleteAction(Request $request, $id)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);
        $object = $this->findOrThrowNotFound($resource, $id, $request);

        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_DELETE, new DataEvent($resource, null));
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_DELETE, new DataEvent($resource, $object));

        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $request->query->set("\$relationships", true);
        $previousObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);
        $request->query->set("\$relationships", true);

        $data = $this->prepareChangeData((object)null, $previousObject, 'Delete');
        $utilsObj = new Utils();
        $utilsObj->sendData($this, [$data]);

        return new Response(null, Response::HTTP_NO_CONTENT, []);

    }
}
