<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\FilesBundle\Controller;

use Ramsey\Uuid\Uuid;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\FilesBundle\Entity\PgFile;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\FilesBundle\Entity\AbstractFile;
use Cybersponse\FilesBundle\Provider\FilesProvider;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class FilesController extends AuthorizationResourceController
{

    /**
     * @return FilesProvider
     */
    private function fileProvider()
    {
        return $this->get('cybersponse.files.provider.files');
    }

    /** @var OwnershipProvider */
    protected $ownershipProvider;
    /**
     * @param OwnershipProvider $ownershipProvider
     */
    public function setOwnershipProvider($ownershipProvider)
    {
        $this->ownershipProvider = $ownershipProvider;
    }
    /** @var TeamsProvider */
    protected $teamProvider;
    /**
     * @param TeamsProvider $teamProvider
     */

    protected $fileProvider;

    public function setFileProvider()
    {
        $this->fileProvider = $this->fileProvider();
    }
    public function setTeamProvider($teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }


    public function uploadFile($request)
    {
        $this->setFileProvider();
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);

        if (!$request->files->has('file')) {
            throw new BadRequestHttpException('No file specified.');
        }

        /** @var UploadedFile $upload */
        $upload = $request->files->get('file');

        if (!$upload->isValid()) {
            throw new BadRequestHttpException($upload->getErrorMessage());
        }
        $fileBody = [];
        $fileBody['filename'] = $upload->getClientOriginalName();
        $fileBody['mimeType'] = $upload->getClientMimeType();
        $fileBody['size'] = $upload->getSize();
        $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
        $fileBody['uploadDate'] = $timestamp;

        $pgFile = new PgFile();
        $container = $this->container->get('doctrine');
        $EntityManager = $container->getManagerForClass(PgFile::class);
        /** @var AbstractFile $file */
        $strm = fopen($upload->getPathname(), 'rb');
        $pgFile->setFile(base64_encode(stream_get_contents($strm)));
        $EntityManager->persist($pgFile);
        $EntityManager->flush();
        $fileBody['file'] = "/api/3/pgfile/" . $pgFile->getUuid();
        $fileBody['uuid'] = Uuid::uuid4()->toString();
        $fileBody['id'] = $fileBody['uuid'];
        return $fileBody;
    }
    public function uploadAction(Request $request)
    {
        $fileBody = $this->uploadFile($request);
        return $this->cpostAction($request, $fileBody);
    }

    public function downloadAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $accept = $request->headers->get('Accept', 'application/json');
        $request->attributes->set('id', $id);
        /** @var AbstractFile $file */
        $file =  parent::findOrThrowNotFound($resource, $id);

        if (strpos($accept, 'application/json') !== false) {
            $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
            return $this->getSuccessResponse($resource, $file);
        } else {
            if ($file->getSize() > 0) {
                $container = $this->container->get('doctrine');
                $EntityManager = $container->getManagerForClass(PgFile::class);
                $EntityRepository = $EntityManager->getRepository(PgFile::class);
                $explodeId = explode('/', $file->getFile());
                $fid = end($explodeId);
                $FileRecord = $EntityRepository->findOneBy(['uuid' => $fid]);
                $response = new Response();
                $response->headers->set('Content-Type', $file->getMimeType());
                $response->headers->set('Content-Disposition', 'attachment; filename="' . $file->getFilename() . '"');
                $response->headers->set('filename', $file->getFilename());
                $response->headers->set('Content-Length', $file->getSize());
                $response->setContent(base64_decode(stream_get_contents($FileRecord->getFile())));

                return $response;
            } else {
                return new Response(null, Response::HTTP_NO_CONTENT);
            }
        }
    }

    /**
     * @param Request $request
     * @param $content
     */
    protected function reinitializeRequest(Request &$request, $content)
    {
        $request->initialize(
            $request->query->all(),
            $request->request->all(),
            $request->attributes->all(),
            $request->cookies->all(),
            $request->files->all(),
            $request->server->all(),
            $content
        );
    }

    /**
     * @param $object object to normalize
     * @return string[] Array of teamIris
     */
    protected function getTeamIris($object)
    {
        if ($object != null) {
            $owners = $object->getOwners();
        }
        $teamIds = array();
        if ($owners != null) {
            foreach ($owners->toArray() as $team) {
                array_push($teamIds, $team->getUuid());
            }
        }
        $teamBaseIri = $this->fileProvider->iriConverter->getIriFromResource($this->fileProvider->teamResource);
        return array_map(function ($teamId) use ($teamBaseIri) {
            return sprintf('%s/%s', $teamBaseIri, $teamId);
        }, $teamIds);
    }
}
