<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Yaml\Dumper as YamlDumper;
use Symfony\Component\Yaml\Parser as YamlParser;

/**
 * Class GeneratePicklistCommand
 */
class GeneratePicklistsCommand extends ContainerAwareCommand
{

    protected function configure()
    {
        $this
            ->setName('cybersponse:metadata:generate:picklists')
            ->setDescription('For Testing only. This will generate UUID\'s for picklist fixtures without uuid\'s specified.')
            ->setDefinition(array(
                new InputOption(
                    'replace',
                    'r',
                    InputOption::VALUE_NONE,
                    'If set, will replace all uuids.'
                ),
            ));
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $replace = true === $input->getOption('replace');

        if ($replace) {
            $confirm = $io->confirm('This will replace the core picklist.yml and picklist_name.yml fixture files with new UUID\'s, are you sure?', false);
            if (!$confirm) {
                $io->writeln('Aborting by user.');
            }
            $io->writeln('Replacing all picklist uuid\'s...');
        } else {
            $io->writeln('Generating missing uuid\'s...');
        }

        $container = $this->getContainer();
        $picklistDir = sprintf('%s/Fixtures/ORM/default/Core/Picklist', $container->getParameter('kernel.root_dir'));

        $picklistFileLocations = [
            sprintf('%s/picklist.yml', $picklistDir),
            sprintf('%s/picklist_name.yml', $picklistDir)
        ];

        $yamlParser = new YamlParser();
        $yamlDumper = new YamlDumper();

        foreach ($picklistFileLocations as $picklistFileLocation) {
            $yamlString = file_get_contents($picklistFileLocation);
            $classes = $yamlParser->parse($yamlString);

            if (empty($classes)) {
                continue;
            }

            foreach ($classes as $class => &$fixtures) {

                $io->writeln(sprintf('Processing %s...', $class));
                $io->progressStart(count($fixtures));
                foreach ($fixtures as &$fixture) {

                    if (!$replace && array_key_exists('uuid', $fixture)) {
                        continue;
                    }

                    $uuid = self::v4(openssl_random_pseudo_bytes(16));;
                    $fixture['uuid'] = $uuid;
                    $io->progressAdvance();
                }
                $io->progressFinish();
            }

            file_put_contents($picklistFileLocation, $yamlDumper->dump($classes, 4));
        }
    }

    static protected function v4($data)
    {
        assert(strlen($data) == 16);

        $data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
        $data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10

        return vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4));
    }
}
