<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputOption;

class MsspSelfEntryCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('cybersponse:mssp:entry')
            ->setDescription('Publish new mapping to elasticsearch')
            ->setHelp("This command allows you to create mapping in elasticsearch based on model metadata by producing a message and sending it to falcon.")
            ->setDefinition([
                new InputOption(
                    'hardwareKey',
                    'hkey',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting hardware key.'
                ),
                new InputOption(
                    'user_id',
                    'usr_id',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting das user_id.'
                ),
                new InputOption(
                    'appliance_id',
                    'appliance',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting das user_id for appliance.'
                ),
                new InputOption(
                    'role',
                    'rle',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting mssp role.'
                )
            ]);
    }

    protected function execute(InputInterface $input, OutputInterface $out)
    {

        $role = $input->getOption('role');
        $user_id = $input->getOption('user_id') ? $input->getOption('user_id') : 'c59e0943-12e1-491f-a041-1071c93311ef';
        $appliance_id = $input->getOption('appliance_id') ? $input->getOption('appliance_id') : '2ad6c11c-1fdd-4973-920f-d1a5b1daed7f';

        if ($role == 'master') {
            $username = 'Tenant';
            $command = sprintf("psql -U cyberpgsql -d venom --command=\"update attribute_metadatas set writeable=False where uuid='d2cf1cd1-44b9-444f-a6dd-d33188bb4fa1';\"");
            exec($command, $output, $errorCode);
            if ($errorCode) {
                $out->writeln(['Disabling Writeable Failed']);
                exit;
            }

            $out->writeln([
                '===========================================================',
                'Disabled Writeable for AllowRemoteMMD for Master Instance',
                '===========================================================',
                '',
            ]);
        } else {
            $username = 'Master';
        }


        $out->writeln([
            '===========================================================',
            'Generating Self Entry Of Tenant',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command="SELECT setval(\'tenants_id_seq\', 2, FALSE);"');
        exec($command, $output, $errorCode);

        // For Tenant Self is always the 1st ID
        $global_seq = 1;
        $uniqId = $input->getOption('hardwareKey');

        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO TENANTS (id, uuid, name, tenantId, active, role, description, ssl, isDedicated, allowremotemmdmodification) VALUES ("' . $global_seq . '", \'b3a700f7-00be-4ef9-90c6-3c8fe6e1be63\', \'Self\',\'"' . $uniqId . '"\', true, \'self\', \'sample\', true, true, true)"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Tenant Id Already Exist']);
            exit;
        } else {
            $out->writeln([
                '===========================================================',
                'Generated Self Entry For Tenant',
                '===========================================================',
                '',
            ]);
        }

        $out->writeln([
            '===========================================================',
            'Adding Owners To Tenant',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO TENANT_TEAM (tenant_uuid, team_uuid) VALUES (\'b3a700f7-00be-4ef9-90c6-3c8fe6e1be63\', \'6e569c09-3bd4-40f1-98b0-cc994464c3c5\')"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Tenant Team CoRelation Failed']);
            exit;
        } else {
            $out->writeln([
                '===========================================================',
                'Tenant Team CoRelation Success',
                '===========================================================',
                '',
            ]);
        }

        $command = sprintf('psql -U cyberpgsql -d venom --command="SELECT setval(\'actors_id_seq\', 5, FALSE);"');
        exec($command, $output, $errorCode);

        $global_seq = 3;

        $out->writeln([
            '===========================================================',
            'Creating Default Remote User',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command=" INSERT INTO ACTORS (id, uuid, createuser, modifyuser, user_id, record_type, firstname, lastname, title, email, phonework) VALUES ("' . $global_seq . '", \'441c0092-8db1-4299-ba1a-6d704465465a\', \'441c0092-8db1-4299-ba1a-6d704465465a\', \'441c0092-8db1-4299-ba1a-6d704465465a\', \'"' . $user_id . '"\', \'person\', \'"' . $username . '"\', \'User\', \'Admin\', \'admin@example.com\', \'646-275-9691\' )"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Remote User Creation Failed']);
            exit;
        }

        $out->writeln([
            '===========================================================',
            'Default Remote User Created',
            '===========================================================',
            '',
        ]);

        $global_seq = 4;

        $out->writeln([
            '===========================================================',
            'Creating Default Appliance User',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command=" INSERT INTO ACTORS (id, uuid, user_id, record_type, name) VALUES ("' . $global_seq . '", \'4312510d-9b52-4fc4-995a-754f76290fb6\', \'"' . $appliance_id . '"\', \'appliance\', \'"' . $username . '"\' )"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Default Appliance Creation Failed']);
        } else {
            $out->writeln([
                '===========================================================',
                'Appliance Created Successfully',
                '===========================================================',
                '',
            ]);
        }

        $out->writeln([
            '===========================================================',
            'Creating Default Remote User Roles',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES (\'441c0092-8db1-4299-ba1a-6d704465465a\', \'17562d60-b841-4ed7-912b-1ebaf063c8c9\' )"');
        exec($command, $output, $errorCode);
        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES (\'441c0092-8db1-4299-ba1a-6d704465465a\', \'21d7a7c6-9d68-438f-b732-e8950c5f745b\' )"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Remote User Roles Creation Failed']);
        } else {
            $out->writeln([
                '===========================================================',
                'Remote User Roles Creation Succeed',
                '===========================================================',
                '',
            ]);
        }

        $out->writeln([
            '===========================================================',
            'Creating Default Remote Appliance Roles',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES (\'4312510d-9b52-4fc4-995a-754f76290fb6\', \'17562d60-b841-4ed7-912b-1ebaf063c8c9\' )"');
        exec($command, $output, $errorCode);
        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES (\'4312510d-9b52-4fc4-995a-754f76290fb6\', \'21d7a7c6-9d68-438f-b732-e8950c5f745b\' )"');
        exec($command, $output, $errorCode);
        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES (\'4312510d-9b52-4fc4-995a-754f76290fb6\', \'c5ff8174-b3fa-41c5-b781-2b890b426a80\' )"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Remote Appliance Roles Creation Failed']);
        } else {
            $out->writeln([
                '===========================================================',
                'Remote Appliance Roles Creation Successed',
                '===========================================================',
                '',
            ]);
        }

        $out->writeln([
            '===========================================================',
            'Assigning Default Remote User Teams',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO TEAM_ACTOR (actor_uuid, team_uuid) VALUES (\'441c0092-8db1-4299-ba1a-6d704465465a\', \'6e569c09-3bd4-40f1-98b0-cc994464c3c5\' )"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Assigning Team to Remote user Failed']);
        } else {
            $out->writeln([
                '===========================================================',
                'Assigned Default Remote User Teams',
                '===========================================================',
                '',
            ]);
        }

        $out->writeln([
            '===========================================================',
            'Assigning Default Appliance Teams',
            '===========================================================',
            '',
        ]);

        $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO TEAM_ACTOR (actor_uuid, team_uuid) VALUES (\'4312510d-9b52-4fc4-995a-754f76290fb6\', \'6e569c09-3bd4-40f1-98b0-cc994464c3c5\' )"');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Assigning Team to Appliance Failed']);
        } else {
            $out->writeln([
                '===========================================================',
                'Assigned Default Remote Appliance Team',
                '===========================================================',
                '',
            ]);
        }

        $out->writeln([
            '===========================================================',
            'Enabling Peer Replicable',
            '===========================================================',
            '',
        ]);

        $role = $input->getOption('role');

        if ($role == "master") {
            $command = sprintf('psql -U cyberpgsql -d venom --command="Update staging_model_metadatas set peerreplicable=true where type not in (\'people\', \'appliances\', \'tenants\', \'routers\');"');
            exec($command, $output, $errorCode);
        } else {
            $command = sprintf('psql -U cyberpgsql -d venom --command="Update staging_model_metadatas set peerreplicable=true where type in (\'alerts\', \'incidents\', \'emails\', \'indicators\');"');
            exec($command, $output, $errorCode);
            if ($errorCode) {
                $out->writeln(['Alerts Peer Replication Failed']);
            }
        }

        if (strcasecmp($role, 'master') == 0) {
            $out->writeln([
                '===========================================================',
                'Adding Remote Execution Playbook Workflow Step',
                '===========================================================',
                '',
            ]);

            $command = sprintf('psql -U cyberpgsql -d venom --command="INSERT INTO workflow_step_types (uuid, collection_uuid, parent_uuid, name, displayname, arguments, icon, background, index, visible, id) VALUES (\'ab3b2e02-5e77-4ed6-8ebd-580f390063a5\', \'f3561053-417f-4591-8fa4-ca9da8e1da8c\', \'ee73e569-2188-43fe-a7f0-1964ba82a4de\', \'RemotePlaybookReference\', \'Reference Remote Playbook\', \'{\"script\": \"/wf/workflow/tasks/remote_workflow_reference\"}\', \'fa fa-sitemap\', \'#fa5b46\', \'425\', true, 200);"');
            exec($command, $output, $errorCode);

            if ($errorCode) {
                $out->writeln(['Remote Reference Playbook Step Insertion Failed']);
            } else {
                $out->writeln([
                    '===========================================================',
                    'Remote Reference Playbook Step Insertion Success',
                    '===========================================================',
                    '',
                ]);
            }
        }

        $out->writeln([
            '===========================================================',
            'Publishing for MSSP Changes',
            '===========================================================',
            '',
        ]);

        $command = sprintf('cd /opt/cyops-api' . PHP_EOL . 'sudo -u nginx php app/console cybersponse:system:update -laf --env=prod');
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Publish For Replication Failed']);
        } else {
            $out->writeln([
                '===========================================================',
                'Peer Replicable Enabled',
                '===========================================================',
                '',
            ]);
        }
    }
}
