<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\TestBundle\Functional;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\AuthorizationBundle\Entity\ModulePermission;
use Cybersponse\AuthorizationBundle\Entity\Role;
use Cybersponse\CoreBundle\Entity\Module;
use Lexik\Bundle\JWTAuthenticationBundle\Security\Authentication\Token\JWTUserToken;
use Liip\FunctionalTestBundle\Test\WebTestCase;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class JwtWebTestCase extends WebTestCase
{
    /** @var array */
    protected $roles = [];

    /** @var Actor */
    protected $actor;

    /**
     * @param array $roles
     */
    protected function setRoles($roles)
    {
        $this->roles = $roles;
    }

    /**
     * @return array
     */
    protected function getRoles()
    {
        return $this->roles;
    }

    protected function createUserToken(Actor $user, $firewallName)
    {
        $token = new JWTUserToken($this->roles);
        foreach ($this->roles as $role) {
            $user->addRole($role);
        }
        $token->setUser($user);
        $token->setAuthenticated(true);
        return $token;
    }

    protected function makeClient($authentication = false, array $params = array())
    {
        $client = parent::makeClient($authentication, $params);

        if ($authentication && isset($this->actor)) {
            $clientContainer = $client->getContainer();

            $this->loginAs($this->actor, 'api');

            /** @var TokenStorageInterface $tokenStorage */
            $tokenStorage = $clientContainer->get('security.token_storage');
            $tokenStorage->setToken($this->createUserToken($this->actor, 'api'));
        }

        return $client;
    }

    /**
     * @param string $expectedModuleName
     * @param bool $canCreate
     * @param bool $canRead
     * @param bool $canUpdate
     * @param bool $canDelete
     * @return Role
     */
    protected function createModulePermission($expectedModuleName, $canCreate = false, $canRead = false, $canUpdate = false, $canDelete = false)
    {
        $module = new Module();
        $module->setType($expectedModuleName);
        $module->setName($expectedModuleName);

        $modulePermissions = new ModulePermission();
        $modulePermissions->setModule($module);
        $modulePermissions->setCanCreate($canCreate);
        $modulePermissions->setCanRead($canRead);
        $modulePermissions->setCanUpdate($canUpdate);
        $modulePermissions->setCanDelete($canDelete);

        $expectedPermission = new Role();
        $expectedPermission->addModulePermission($modulePermissions);

        return $expectedPermission;
    }
}
