<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\UtilsBundle;

use Dunglas\ApiBundle\JsonLd\Response;
use Dunglas\ApiBundle\Exception\InvalidArgumentException;
use Symfony\Component\HttpKernel\Exception\HttpException;


class ARCUtils
{

    const LINK_KEY = '__link';
    const DE_LINK_KEY = '__unlink';
    const AS_JSON_KEY = "\$asJson";
    const RELATIONSHIP_KEY = "\$relationships";

    public static function setAsJson($request, $set_asJson = false)
    {
        if (!$request->query->has(self::AS_JSON_KEY) && !$set_asJson) {
            $request->query->set(self::AS_JSON_KEY, true);
            $set_asJson = true;
        } else if ($set_asJson) {
            $request->query->remove(self::AS_JSON_KEY, true);
            $set_asJson = false;
        }
        return $set_asJson;
    }
    /**
     * @param $request
     * @param bool $set_relation
     * @return bool
     */
    public static function setRelationShip($request, $set_relation = false)
    {
        if (!$request->query->has(self::RELATIONSHIP_KEY) && !$set_relation) {
            $request->query->set(self::RELATIONSHIP_KEY, true);
            $set_relation = true;
        } else if ($set_relation) {
            $request->query->remove(self::RELATIONSHIP_KEY, true);
            $set_relation = false;
        }
        return $set_relation;
    }

    /**
     * @param $request_body
     * @return mixed
     */
    public static function updateReferenceToIriAndCheckForRelation($request_body)
    {
        foreach ($request_body as $key => $value) {
            if (gettype($value) == "object" && isset($request_body->$key->{'@id'})) {
                $request_body->$key = $request_body->$key->{'@id'};
            }
        }
    }

    /**
     * @param $request_body
     */
    public static function validateRequestData($request_body)
    {
        if (gettype($request_body) != 'object') {
            throw new InvalidArgumentException('Invalid request body. valid Json object request body expected.');
        }
    }

    /**
     * @param $previousObject
     * @param $updatedObject
     * @return null
     */
    public static function extractEntityType($previousObject, $updatedObject, $identity)
    {
        if (array_key_exists($identity, $updatedObject)) {
            return $updatedObject[$identity];
        } else if (array_key_exists($identity, $previousObject)) {
            return $previousObject[$identity];
        }
        return null;
    }

    /**
     * @param $teams
     * @return array
     */
    public static function getUuidFromIRI($teams)
    {
        $objectTeam = [];
        foreach ($teams as $team) {
            $teamId = explode('/', $team);
            array_push($objectTeam, end($teamId));
        }
        return $objectTeam;
    }

    public static function validateRequestKeys($key, $object)
    {
        if (!array_key_exists($key, $object)) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $key . ' key does not exist in object');
        }
    }

    // FOR POST REQUEST MERGE LINK AND UNLINK INTO MAIN REQUEST

    /**
     * @param $reqObject
     * @param $preObj
     * @return mixed
     */
    public static function mergePostRequest($reqObject)
    {
        $linkObj = isset($reqObject->{self::LINK_KEY}) ? $reqObject->{self::LINK_KEY} : (object) null;
        unset($reqObject->{self::LINK_KEY});
        unset($reqObject->{self::DE_LINK_KEY});
        return self::mergeLinkPost($reqObject, $linkObj);
    }

    public function mergeLinkPost($reqObject, $linkObj)
    {
        foreach ($linkObj as $key => $value) {
            if (!is_array($value)){
                $value = explode(',', $value);
            }
            $referenceToIriObj = [];
            foreach ($value as $innerVal) {
                if (is_object($innerVal) && $innerVal->{'@id'}) {
                    array_push($referenceToIriObj, $innerVal->{'@id'});
                } else {
                    array_push($referenceToIriObj, $innerVal);
                }
            }
            if (count($referenceToIriObj)) {
                $reqObject->$key = $referenceToIriObj;
            }
        }
        return $reqObject;
    }

    /**
     * @param $reqObject
     * @param $preObj
     * @return mixed
     */
    public static function mergeRequest($reqObject, $preObj)
    {
        $linkObj = isset($reqObject[self::LINK_KEY]) ? $reqObject[self::LINK_KEY] : (object) null;
        $unlinkObj = isset($reqObject[self::DE_LINK_KEY]) ? $reqObject[self::DE_LINK_KEY] : (object) null;
        unset($reqObject[self::LINK_KEY]);
        unset($reqObject[self::DE_LINK_KEY]);
        return self::mergeLinkAndUnlink($reqObject, $preObj, $linkObj, $unlinkObj);
    }

    /**
     * @param $reqObject
     * @param $preObj
     * @param $linkObj
     * @param $unlinkObj
     * @return mixed
     */
    public static function mergeLinkAndUnlink($reqObject, $preObj, $linkObj = [], $unlinkObj = [])
    {
        // Handling Linked Object
        foreach ($linkObj as $key => $value) {
            $updatedReq[$key] = [];
            self::validateRequestKeys($key, $preObj);

            // Check if Link object has been send as an string or object then convert to array
            if ($linkObj[$key] && is_string($linkObj[$key])) {
                $linkObj[$key] = array_map('trim', explode(',', $linkObj[$key]));
            } else if (is_object($linkObj[$key])) {
                $linkObj[$key] = [$linkObj[$key]];
            }

            if (count($preObj[$key])) {
                $updatedReq[$key] = $preObj[$key];
            }

            foreach ($linkObj[$key] as $innerKey) {
                if (gettype($innerKey) == 'array' && array_key_exists('@id', $innerKey)) {
                    array_push($updatedReq[$key], $innerKey['@id']);
                } else {
                    array_push($updatedReq[$key], $innerKey);
                }
            }
            if (gettype($updatedReq[$key]) == "array") {
                $reqObject[$key] = array_map('unserialize', array_unique(array_map('serialize', $updatedReq[$key])));
            } else {
                $reqObject[$key] = array_values(array_unique($updatedReq[$key]));
            }
        }

        // Handling Unlink Object
        foreach ($unlinkObj as $key => $value) {
            self::validateRequestKeys($key, $preObj);
            if (isset($reqObject[$key])) {
                $reqObject[$key] = array_values(array_diff($reqObject[$key], $unlinkObj[$key]));
            } else {
                $reqObject[$key] = array_values(array_diff($preObj[$key], $unlinkObj[$key]));
            }
        }
        return $reqObject;
    }

    public static function extractRequestBody($request, $file)
    {
        if ($file !== null) {
            return json_decode(json_encode($file));
        } else {
            $request_body = json_decode($request->getContent());
            self::updateReferenceToIriAndCheckForRelation($request_body);
            return $request_body;
        }
    }
}
